/***************************
* Creator : PioDiamond
* Shader : PioShader Vanilla
* Edition : New Year 2020
* Target : Low-end Device
*
*
* Youtube : https://youtube.com/PioDiamond
* Instagram : https://instagram.com/PioDiamond
* 
*
* 
* Reference
** Noise : https://www.shadertoy.com/view/Xs2fDw
***************************/

// Varying

varying vec4 position;
varying vec4 worldPos;
varying vec4 fogColor;
varying vec4 view;
UNIFORM vec4 TILE_LIGHT_COLOR;

// Struct

struct kondisi {
  float siang;
  float hujan;
  float gelap;
} deteksi;

// Function

float distance_( float i ){
  return min( length( worldPos.xyz ) / RENDER_DISTANCE * i, 1.0 );
}

// Noise Source : https://gist.github.com/joelpryde/5224151

highp float hash( highp float n )
{
	return fract(sin(n)*43758.121);
}

highp float noise( highp vec2 x )
{
	highp vec2 p = floor(x);
	highp vec2 f = fract(x);
    	f = f*f*(3.0-2.0*f);
    	highp float n = p.x + p.y*57.0;
    	highp float res = mix(mix( hash(n+  0.0), hash(n+  1.0),f.x), mix( hash(n+ 57.0), hash(n+ 58.0),f.x),f.y);
    	return res;
}

bool is_water(){
  bool water = false;
  #if !defined( SEASONS ) && defined( BLEND )
    if( color.b > color.r )
      water = true;
  #endif
  return water;
}

bool is_plant(){
 if( color.a == 0. )
   return true;
 #ifdef ALPHA_TEST
   if( color.g > color. r )
     return true;
 #endif
 return false;
}

#ifdef is_fragment // ---------------------------

void deteksi_(){
  float cahaya = texture2D( TEXTURE_1, vec2( 0.0, 1.0 )).r;
  cahaya = max( cahaya * 2.0 - 1.0, 0.0 );
  float uvy = max( uv1.y - 0.8, 0.0 ) * 5.;

  deteksi.siang = cahaya;
  deteksi.gelap = 1.0 - max( min( uvy, cahaya ), uv1.x );
  deteksi.hujan = 1.0 - pow( FOG_CONTROL.y, 11.0 );
}

vec4 shadow_block( vec4 c ){
	if( fract( position.y ) != 0.5 && color.b != color.r && color.r != color.g ){
  	vec4 shadow  = vec4( 0.5 * c.rgb, c.a );
  	return mix( c, shadow, deteksi.gelap );
	} else {
		vec4 shadow  = vec4( 0.7 * c.rgb, c.a );
  	return mix( c, shadow, deteksi.gelap );
	}
}

highp vec4 water_waves( vec4 c ){
  highp vec2 pos_water;
  	pos_water = position.zx * 2.5;
	
  highp float time = TIME * .9;
	pos_water *= vec2( 0.4,  2.0 );
  pos_water *= mat2( -.7,  -.7, .7,  -.7 );
	pos_water.x += time;
	highp float water_noise = noise( pos_water );
	pos_water.x -= time;
	water_noise += noise( pos_water );
	water_noise /= 4.0;
  float water_distance = distance_( 1.8 );

  vec4 diffuse = color;
	vec3 col_water = vec3( color.rgb ) + water_noise * 0.5;
  diffuse.rgb = min( col_water, 1.0 );
  diffuse.a = ( water_noise * 0.5 ) + water_distance;
  diffuse.a = min( diffuse.a, 1.0 );
  diffuse = mix( diffuse, c,  water_distance);
  return diffuse;
}

float inrect(vec2 pos, float x1, float y1, float x2, float y2, float focus){
 return min(1.0, max(min(min(pos.x - x1, x2 - pos.x), min(pos.y - y1, y2 - pos.y)), 0.0) / focus);
}

vec4 Player_Shadow(){
   vec2 uv = uv1;
   vec3 lookvector = worldPos.xyz;
   lookvector.x*=2.0;
   vec3 pos = lookvector + vec3(0.4, 0.4, 0.4); 
   vec3 dir = vec3(-1.0, (1.25) * 0.31, 0.0);
   float factor = 1.0;
   if ( pos.x < 0.2 ){
     factor = max( 0.0, pos.x / 0.4 + 0.5 );
   }
   pos += dir * pos.x;
   float focus = .04;
   float footwalk = sin(( position.x - lookvector.x ) * 2.0 + ( position.z - lookvector.z ) * 2.0);
   float handswalk = sin(( position.x - lookvector.x ) * 2.0 + ( position.z - lookvector.z )) * .5;
   pos.yz -= vec2( .2, .4 );
   float body = max(inrect(pos.yz, -1.5 + footwalk*.4, -0.25, 0.75, .1, focus), inrect(pos.yz, -1.5 - footwalk*.4, -.1, 0.75, 0.25, focus));
   float hands = max(inrect(pos.yz, -0.5 + footwalk * .1, -0.5, 0.25, .1, focus), inrect(pos.yz, -0.5 - footwalk * 0.1, -.1, 0.25, 0.5, focus));
   
   uv.y -= min(1.0, max(body, hands)) * factor * 0.4;
   return texture2D( TEXTURE_1, uv );
}

vec4 Dynamic_Light(){
   vec2 uv = uv1;
	 float cahaya = mix( 0.0, uv1.x, distance_( 0.0 ) );
   uv.x = mix( uv1.x, cahaya, max(TILE_LIGHT_COLOR.x - .0, 0.0) * 0.0);
   return texture2D( TEXTURE_1, uv );
}

vec4 Player_mode(){
   return mix( Player_Shadow(), Dynamic_Light(), deteksi.gelap );
}

#endif




