/***************************
* Creator : PioDiamond
* Shader : PioShader Vanilla
* Edition : New Year 2020
* Target : Low-end Device
*
*
* Youtube : https://youtube.com/PioDiamond
* Instagram : https://instagram.com/PioDiamond
* 
*
* 
* Reference
** Noise : https://www.shadertoy.com/view/Xs2fDw
***************************/

bool is_water( in PS_Input PSInput ){
  bool water = false;
  #if !defined( SEASONS ) && defined( BLEND )
    if( PSInput.color.b > PSInput.color.r )
      water = true;
  #endif
  return water;
}

bool is_plant( in PS_Input PSInput ){
 if( PSInput.color.a == 0. )
   return true;
 #ifdef ALPHA_TEST
   if( PSInput.color.g > PSInput.color. r )
     return true;
 #endif
 return false;
}

float distance_( in float i, in PS_Input PSInput ){
  return min( length( PSInput.worldPos.xyz ) / PSInput.RENDER_DISTANCE * i, 1.0 );
}

// Noise Source : https://gist.github.com/joelpryde/5224151

float hash( float n )
{
	return frac(sin(n)*43758.121);
}

float noise( float2 x )
{
	float2 p = floor(x);
	float2 f = frac(x);
    	f = f*f*(3.0-2.0*f);
	float n = p.x + p.y*57.0;
	float res = lerp(lerp( hash(n), hash(n + 1.0), f.x), lerp( hash(n + 57.0), hash(n + 58.0),f.x),f.y);
	return res;
}

#ifdef fragment
struct kondisi {
  float siang;
  float hujan;
  float gelap;
};

void deteksi_( in PS_Input PSInput, out kondisi deteksi ){
  float cahaya = TEXTURE_1.Sample( TextureSampler1, float2( 0.0, 1.0 )).r;
  cahaya = max( cahaya * 2.0 - 1.0, 0.0 );
  float uvy = max( PSInput.uv1.y - 0.8, 0.0 ) * 5.;

  deteksi.siang = cahaya;
  deteksi.gelap = 1.0 - max( min( uvy, cahaya ), PSInput.uv1.x );
  deteksi.hujan = 1.0 - pow( PSInput.fogControl.y, 11.0 );
}

void shadow_block( inout float4 c, in PS_Input PSInput, in kondisi deteksi ){
	float3 shadow = c.rgb;
	if( frac( PSInput.pos.y ) != 0.5 && PSInput.color.b != PSInput.color.r && PSInput.color.r != PSInput.color.g ){
  		shadow  = 0.5 * c.rgb;
  	} else {
		shadow  = 0.7 * c.rgb;
  	}
  	c.rgb = lerp( c.rgb, shadow, deteksi.gelap );
}

void water_waves( inout float4 c, in PS_Input PSInput ){
	float2 pos_water;
	pos_water = PSInput.pos.zx * 2.5;

	float time = TIME * .9;
	pos_water *= float2( 0.4,  2.0 );
	pos_water = mul( pos_water, float2x2( -.7,  -.7, .7,  -.7 ) );
	pos_water.x += time;
	float water_noise = noise( pos_water );
	pos_water.x -= time;
	water_noise += noise( pos_water );
	water_noise /= 4.0;
	float water_distance = distance_( 1.8, PSInput );

	float4 diffuse = PSInput.color;
	float3 col_water = float3( PSInput.color.rgb ) + water_noise * 0.5;
	diffuse.rgb = min( col_water, 1.0 );
	diffuse.a = ( water_noise * 0.5 ) + water_distance;
	diffuse.a = min( diffuse.a, 1.0 );
	diffuse = lerp( diffuse, c,  water_distance);
	c = diffuse;
}

float inrect(float2 pos, float x1, float y1, float x2, float y2, float focus){
	return min(1.0, max(min(min(pos.x - x1, x2 - pos.x), min(pos.y - y1, y2 - pos.y)), 0.0) / focus);
}

float4 Player_Shadow( in PS_Input PSInput ){
   float2 uv = PSInput.uv1;
   float3 lookvector = PSInput.worldPos.xyz;
   lookvector.x *= 2.0;
   float3 pos = lookvector + float3(0.4, 0.4, 0.4); 
   float3 dir = float3(-1.0, (1.25) * 0.31, 0.0);
   float factor = 1.0;
   if ( pos.x < 0.2 ){
     factor = max( 0.0, pos.x / 0.4 + 0.5 );
   }
   pos += dir * pos.x;
   float focus = .04;
   float footwalk = sin(( PSInput.pos.x - lookvector.x ) * 2.0 + ( PSInput.pos.z - lookvector.z ) * 2.0);
   float handswalk = sin(( PSInput.pos.x - lookvector.x ) * 2.0 + ( PSInput.pos.z - lookvector.z )) * .5;
   pos.yz -= float2( .2, .4 );
   float body = max(inrect(pos.yz, -1.5 + footwalk*.4, -0.25, 0.75, .1, focus), inrect(pos.yz, -1.5 - footwalk*.4, -.1, 0.75, 0.25, focus));
   float hands = max(inrect(pos.yz, -0.5 + footwalk * .1, -0.5, 0.25, .1, focus), inrect(pos.yz, -0.5 - footwalk * 0.1, -.1, 0.25, 0.5, focus));
   
   uv.y -= min(1.0, max(body, hands)) * factor * 0.4;
   return TEXTURE_1.Sample( TextureSampler1, uv );
}

float4 Dynamic_Light( in PS_Input PSInput ){
   float2 uv = PSInput.uv1;
	 float cahaya = lerp( 1.0, PSInput.uv1.x, distance_( 16.0, PSInput ) );
   uv.x = lerp( PSInput.uv1.x, cahaya, max(TILE_LIGHT_COLOR.x - .9, 0.0) * 10.0);
   return TEXTURE_1.Sample( TextureSampler1, uv );
}

float4 Player_mode( in PS_Input PSInput, in kondisi deteksi ){
   return lerp( Player_Shadow( PSInput ), Dynamic_Light( PSInput ), deteksi.gelap );
}
#endif